@@ -1,6 +1,6 @@
 from threading import Thread
 import inspect
-import traceback
+from kafkacrypto.utils import format_exception_shim
 import pysodium
 from time import time,sleep
 import msgpack
@@ -12,6 +12,7 @@ from kafkacrypto.exceptions import KafkaCryptoError, KafkaCryptoSerializeError
 from kafkacrypto.message import KafkaCryptoMessage
 from kafkacrypto.ratchet import Ratchet
 from kafkacrypto.keygenerator import KeyGenerator
+from kafkacrypto.utils import log_limited
 
 class KafkaCrypto(KafkaCryptoBase):
   """Class handling the sending and receiving of encrypted messages.
@@ -105,19 +106,22 @@ class KafkaCrypto(KafkaCryptoBase):
 
     # Add management frame TPS
     for ntp in [self.MGMT_TOPIC_CHAINS, self.MGMT_TOPIC_ALLOWLIST, self.MGMT_TOPIC_DENYLIST]:
-      self._tps[ntp] = TopicPartition(ntp.decode('utf-8'),0)
+      self._tps[ntp] = TopicPartition(ntp,0)
       self._tps_offsets[ntp] = 0
       self._tps_updated = True
 
     kvs = self._cryptostore.load_opaque_value('oldkeys',section="crypto")
     if not (kvs is None):
-      kvs = msgpack.unpackb(kvs)
+      kvs = msgpack.unpackb(kvs,raw=True)
       if 'pgens' in kvs.keys() or b'pgens' in kvs.keys():
         self._logger.info("Found pgens to load.")
         t = 'pgens'
         if not (t in kvs.keys()):
           t = b'pgens'
         for [root,ki,ks,b] in kvs[t]:
+          if isinstance(root,(bytes,bytearray)):
+            self._logger.debug("loaded a pgen root in bytes (should be string)")
+            root = root.decode('utf-8')
           self._logger.info("Attempting load of root=%s, ki=%s, b=%s at %s", root, ki, b, time())
           if time()-b < self.CRYPTO_MAX_PGEN_AGE:
             if not (root in self._pgens.keys()):
@@ -145,15 +149,16 @@ class KafkaCrypto(KafkaCryptoBase):
         self._lock.acquire()
         for msg in msgset:
           topic = msg.topic
-          if (isinstance(topic,(str,))):
-            topic = topic.encode('utf-8')
+          if (isinstance(topic,(bytes,bytearray))):
+            self._logger.debug("passed a topic in bytes (should be string)")
+            topic = topic.decode('utf-8')
           self._tps_offsets[topic] = msg.offset+1
           self._logger.debug("Processing message: %s", msg)
           if topic[-len(self.TOPIC_SUFFIX_REQS):] == self.TOPIC_SUFFIX_REQS:
             root = topic[:-len(self.TOPIC_SUFFIX_REQS)]
             # A new receiver: send all requested keys
             try:
-              kreq = msgpack.unpackb(msg.key)
+              kreq = msgpack.unpackb(msg.key,raw=True)
               if root in self._pgens.keys():
                 ki = []
                 s = []
@@ -162,17 +167,17 @@ class KafkaCrypto(KafkaCryptoBase):
                     ki.append(ski)
                     s.append(sk['secret'])
                 if len(ki) > 0:
-                  k = msgpack.packb(ki)
+                  k = msgpack.packb(ki, use_bin_type=True)
                   v = self._cryptoexchange.encrypt_keys(ki, s, root, msgval=msg.value)
                   if not (v is None):
                     self._logger.info("Sending current encryption keys for root=%s to new receiver, msgkey=%s.", root, k)
-                    self._kp.send((root + self.TOPIC_SUFFIX_KEYS).decode('utf-8'), key=k, value=v)
+                    self._kp.send(root + self.TOPIC_SUFFIX_KEYS, key=k, value=v)
                   else:
                     self._logger.info("Failed sending current encryption keys for root=%s to new receiver.", root)
                 else:
                   self._logger.info("No keys for root=%s to send to new receiver.", root)
             except Exception as e:
-              self._parent._logger.warning("".join(traceback.format_exception(etype=type(e), value=e, tb=e.__traceback__)))
+              self._parent._logger.warning("".join(format_exception_shim(etype=type(e), value=e, tb=e.__traceback__)))
           elif topic[-len(self.TOPIC_SUFFIX_KEYS):] == self.TOPIC_SUFFIX_KEYS:
             root = topic[:-len(self.TOPIC_SUFFIX_KEYS)]
             # New key(s)
@@ -219,7 +224,7 @@ class KafkaCrypto(KafkaCryptoBase):
        	      self._cryptostore.store_value(c,deny,section='denylist')
           else:
             # unknown object
-            self._logger.warning("Unknown topic type in message: %s", msg)
+            log_limited(self._logger.warning, "Unknown topic type in message: %s", msg)
         self._lock.release()
   
       # Flush producer
@@ -256,14 +261,14 @@ class KafkaCrypto(KafkaCryptoBase):
           kis = list(self._cwaits[root].keys())
           if len(kis) > 0:
             if (not (root in self._subs_last.keys()) or self._subs_last[root][0]+self.CRYPTO_SUB_INTERVAL<time()):
-              k = msgpack.packb(kis)
+              k = msgpack.packb(kis, use_bin_type=True)
               v = self._cryptoexchange.signed_epk(root)
               if not (k is None) and not (v is None):
                 self._logger.info("Sending new subscribe request for root=%s, msgkey=%s", root, k)
-                self._kp.send((root + self.TOPIC_SUFFIX_SUBS).decode('utf-8'), key=k, value=v)
+                self._kp.send(root + self.TOPIC_SUFFIX_SUBS, key=k, value=v)
                 if not self._cryptoexchange.valid_spk_chain():
                   # if using default/temp ROT, send directly as well
-                  self._kp.send((root + self.TOPIC_SUFFIX_REQS).decode('utf-8'), key=k, value=v)
+                  self._kp.send(root + self.TOPIC_SUFFIX_REQS, key=k, value=v)
                 self._subs_last[root] = [time(),kis]
               else:
                 self._logger.info("Failed to send new subscribe request for root=%s", root)
@@ -312,7 +317,7 @@ class KafkaCrypto(KafkaCryptoBase):
             # stored pgens do not have generators as they should never be used for active production
             # (but secret stays around so lost consumers can catch up)
         self._logger.info("Saving %s old production keys.", len(kvs['pgens']))
-        self._cryptostore.store_opaque_value('oldkeys',msgpack.packb(kvs),section="crypto")
+        self._cryptostore.store_opaque_value('oldkeys',msgpack.packb(kvs, use_bin_type=True),section="crypto")
       self._lock.release()
   
       # Finally, loop back to poll again
@@ -328,8 +333,9 @@ class KafkaCrypto(KafkaCryptoBase):
       self._parent = parent
 
     def serialize(self, topic, value):
-      if (isinstance(topic,(str,))):
-        topic = topic.encode('utf-8')
+      if (isinstance(topic,(bytes,bytearray))):
+        self._parent._logger.debug("passed a topic in bytes (should be string)")
+        topic = topic.decode('utf-8')
       if value is None:
         return None
       if (isinstance(value,(KafkaCryptoMessage,))):
@@ -355,9 +361,9 @@ class KafkaCrypto(KafkaCryptoBase):
         gen = pgen[self._kv]
         salt = gen.salt()
         key,nonce = gen.generate()
-        msg = b'\x01' + msgpack.packb([keyidx,salt,pysodium.crypto_secretbox(value,nonce,key)])
+        msg = b'\x01' + msgpack.packb([keyidx,salt,pysodium.crypto_secretbox(value,nonce,key)], use_bin_type=True)
       except Exception as e:
-        self._parent._logger.warning("".join(traceback.format_exception(etype=type(e), value=e, tb=e.__traceback__)))
+        self._parent._logger.warning("".join(format_exception_shim(etype=type(e), value=e, tb=e.__traceback__)))
       finally:
         self._parent._lock.release()
       return msg
@@ -370,19 +376,20 @@ class KafkaCrypto(KafkaCryptoBase):
       self.WAIT_INTERVAL = key_wait_interval
 
     def deserialize(self, topic, bytes_):
-      if (isinstance(topic,(str,))):
-        topic = topic.encode('utf-8')
+      if (isinstance(topic,(bytes,bytearray))):
+        self._parent._logger.debug("passed a topic in bytes (should be string)")
+        topic = topic.decode('utf-8')
       if bytes_ is None:
         return None
       root = self._parent.get_root(topic)
       if len(bytes_) < 1 or bytes_[0] != 1:
         return KafkaCryptoMessage.fromBytes(bytes_,deser=self,topic=topic)
       try:
-        msg = msgpack.unpackb(bytes_[1:])
+        msg = msgpack.unpackb(bytes_[1:],raw=True)
         if (len(msg) != 3):
           raise KafkaCryptoSerializeError("Malformed Message!")
       except Exception as e:
-        self._parent._logger.debug("".join(traceback.format_exception(etype=type(e), value=e, tb=e.__traceback__)))
+        self._parent._logger.debug("".join(format_exception_shim(etype=type(e), value=e, tb=e.__traceback__)))
         return KafkaCryptoMessage.fromBytes(bytes_,deser=self,topic=topic)
       ki = msg[0]
       salt = msg[1]
@@ -408,7 +415,7 @@ class KafkaCrypto(KafkaCryptoBase):
             i = self.MAX_WAIT_INTERVALS
        	  ntp =	(root+self._parent.TOPIC_SUFFIX_KEYS)
        	  if not (ntp in self._parent._tps):
-       	    self._parent._tps[ntp] = TopicPartition(ntp.decode('utf-8'),0)
+       	    self._parent._tps[ntp] = TopicPartition(ntp,0)
             self._parent._tps_offsets[ntp] = 0
             self._parent._tps_updated = True
           else:
@@ -441,6 +448,9 @@ class KafkaCrypto(KafkaCryptoBase):
     return self.Deserializer(self,'value',max_key_wait_intervals,key_wait_interval)
 
   def get_producer(self,root):
+    if isinstance(root,(bytes,bytearray)):
+      self._logger.debug("passed a root in bytes (should be string)")
+      root = root.decode('utf-8')
     if (self.MGMT_LONG_KEYINDEX == True):
       ki,ks,kg,vg = self._seed.get_key_value_generators(root, node=self._cryptokey.get_spk())
     else:
@@ -460,7 +470,7 @@ class KafkaCrypto(KafkaCryptoBase):
       self._pgens[root][ki]['birth'] = time()
       ntp = (root+self.TOPIC_SUFFIX_REQS)
       if not (ntp in self._tps):
-        self._tps[ntp] = TopicPartition(ntp.decode('utf-8'),0)
+        self._tps[ntp] = TopicPartition(ntp,0)
         self._tps_offsets[ntp] = 0
         self._tps_updated = True
       self._logger.info("Got new producer key for root=%s. New key index=%s", root, ki)
