@@ -1,9 +1,9 @@
 from time import time
-import traceback
+from kafkacrypto.utils import format_exception_shim
 import pysodium
 import msgpack
 import logging
-from kafkacrypto.chain import process_chain
+from kafkacrypto.chain import process_chain, ProcessChainError
 
 class Provisioners(object):
   """Class validating key requests
@@ -13,12 +13,14 @@ class Provisioners(object):
           denylist (array): Optional list of denylisted public keys
   """
   def __init__(self, allowlist=None, denylist=None):
+    self._logger = logging.getLogger(__name__)
     self.__allowlist = allowlist
     self.__denylist = denylist
 
   def reencrypt_request(self, topic, cryptoexchange, msgkey=None, msgval=None):
-    if (isinstance(topic,(str))):
-      topic = bytes(topic, 'utf-8')
+    if (isinstance(topic,(bytes,bytearray))):
+      self._logger.debug("topic provided as bytes (should be string)")
+      topic = topic.decode('utf-8')
     #
     # msgval should be a msgpacked chain chaining to a provisioner.
     # The last item in the array is the encrypted public key to retransmit.
@@ -26,13 +28,18 @@ class Provisioners(object):
     try:
       pk = None
       try:
-        pk = process_chain(msgval,topic,b'key-encrypt-subscribe',allowlist=self.__allowlist,denylist=self.__denylist)
+        pk,pkprint = process_chain(msgval,topic,'key-encrypt-subscribe',allowlist=self.__allowlist,denylist=self.__denylist)
+      except ProcessChainError as pce:
+        raise pce
       except:
         pass
       if (pk is None):
-        raise ValueError("Request did not validate.")
+        if not (pkprint is None):
+          raise ProcessChainError("Request did not validate: ", pkprint)
+        else:
+          raise ValueError("Request did not validate!")
       msg = cryptoexchange.signed_epk(topic, epk=pk[2])
     except Exception as e:
-      logging.warning("".join(traceback.format_exception(etype=type(e), value=e, tb=e.__traceback__)))
+      self._logger.warning("".join(format_exception_shim(etype=type(e), value=e, tb=e.__traceback__)))
       return (None, None)
     return (msgkey, msg)
